﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Net;
using System.IO;

using System.Web; //Add reference to System.Web
using System.Runtime.Serialization.Json; //Add reference to System.Runtime.Serialization

namespace RedCritterConnecter.Samples
{
    //Create a class to contain the response
    public class AwardBadgeResponse
    {
        public String ErrorMessage { get; set; }
        public Boolean Submitted { get; set; }
        public String RequestID { get; set; }
        public String APIVersion { get; set; }
        public String ReturnParam1 { get; set; }
        public String ReturnParam2 { get; set; }
        public String ReturnParam3 { get; set; }
        private Int32 ErrorCode { get; set; }
    }

    public class AwardBadge
    {

        public delegate void OnAwardBadgeResponse(AwardBadgeResponse AwardBadgeResponse);
        public event OnAwardBadgeResponse onAwardBadgeResponse;

        public delegate void OnAwardBadgeResponseError(AwardBadgeResponse AwardBadgeResponse);
        public event OnAwardBadgeResponseError onAwardBadgeResponseError;

        const string CONST_ConnecterBaseURL = "https://www.redcritterconnecter.com/";
                
        public void Execute(String SecretKey, String ExternalAccountID, String ProfileEntity, String BadgeName)
        {
            try
            {
                //Create url encoded parameters in query string
                String queryString = "secretkey=" + System.Web.HttpUtility.UrlEncode(SecretKey) + "&externalaccountid=" + System.Web.HttpUtility.UrlEncode(ExternalAccountID) + "&badgename=" + System.Web.HttpUtility.UrlEncode(BadgeName) + "&profileentity=" + System.Web.HttpUtility.UrlEncode(ProfileEntity);

                //Create a new instance of a WebClient
                WebClient wc = new System.Net.WebClient();

                //Prevent this request from caching in order to ensure that it is sent to server
                wc.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.NoCacheNoStore);

                //Attach an event handler to receive the response
                wc.DownloadStringCompleted += onAwardBadgeResponseReceived;

                //Make the call
                wc.DownloadStringAsync(new Uri(CONST_ConnecterBaseURL + "services/gamificationv1/AwardBadge?" + queryString, UriKind.Absolute));
            }
            catch
            {
                //Something went wrong communicating with the server
                if (onAwardBadgeResponseError != null)
                {
                    onAwardBadgeResponseError(null);
                }
            }

        }

        private void onAwardBadgeResponseReceived(object sender, DownloadStringCompletedEventArgs e)
        {
            try
            {
                //Create a JSON serializer
                System.Runtime.Serialization.Json.DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(AwardBadgeResponse));

                //Copy the string into a memory stream
                MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(e.Result));

                //Read the stream into an object matching the response's type
                AwardBadgeResponse AwardBadgeResponse = (AwardBadgeResponse)s.ReadObject(ms);

                //Work with the populated response object
                if (AwardBadgeResponse.Submitted == true)
                {
                    //Request was accepted, raise the success event
                    onAwardBadgeResponse(AwardBadgeResponse);
                }
                else
                {
                    //Something went wrong
                    if (onAwardBadgeResponseError != null)
                    {
                        //Request failed, raise the error event
                        onAwardBadgeResponseError(AwardBadgeResponse);
                    }
                }
            }
            catch
            {
                //Something went wrong
                onAwardBadgeResponseError(null);
            }
        }
    }
}
